==> aprs4r-0.8.1.tar.gz: aprs4r-0.8.1/src/aprs4r/SocketManager.rb <==
#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'Logger'

require 'aprs4r/AX25Socket'
require 'aprs4r/ISSocket'
require 'aprs4r/RawSocket'
require 'aprs4r/SocketThread'


class SocketManager

  @@singleton = nil

  @@threads = Array.new
  @@sockets = Hash.new
  @@listeners = Hash.new
  @@duplicates = Hash.new

  @@logger = Logger.get_logger( "SocketManager")


  def SocketManager.init( configuration)
    @@logger.info( "init( configuration)")

    sockets = configuration.devices
    sockets.each { |name, configuration|

      if configuration.enable 

        @@logger.debug( "name: #{name}")
        @@logger.debug( "configuration: #{configuration}")
        
        socket = nil

        # FIXME: dynamic class loading
        if name == "is0"
          socket = ISSocket.new( configuration)
        elsif name == "rf0"
          socket = AX25Socket.new( configuration)
        end
        
        @@logger.debug( "socket: #{socket}")
        
        @@threads << Thread.new { 
          thread = SocketThread.new( socket)
          thread.run
        }
        
        @@sockets[name] = socket
        @@listeners[name] = Array.new
        @@duplicates[name] = Hash.new
        
      end
    }

    return
  end

  
  def SocketManager.addListener( name, listener)
    @@logger.info( "addListener( name, listener)")
    
    if @@sockets.key?( name) 
      listeners = @@listeners[name]
      listeners << listener
    end

    return
  end


  def SocketManager.removeListener( name, listener)
    @@logger.info( "removeListener( name, listener)")

    if @@sockets.key?( name)
      listeners = @@listeners[name]
      listeners.delete( listener)
    end

    return
  end

  
  def SocketManager.receiveAPRSMessage( name, message)
    @@logger.info( "receiveAPRSMessage( name, message)")

    if message.nil?
      return
    end

    if SocketManager.duplicate?( name, message)
      @@logger.warn( "recv: duplicate message dropped")
      return
    end

    @@logger.warn( "message recv on #{name}: #{message}")

    # deliver message to all registered listeners
    listeners = @@listeners[name]

    if ! listeners.nil?
      listeners.each{ |listener|
        listener.receiveAPRSMessage( message.clone)
      }
    end

    return
  end

  
  def SocketManager.sendAPRSMessage( name, message)
    @@logger.info( "sendAPRSMessage( name, message)")
    
    socket = @@sockets[name]

    if message.nil? || socket.nil?
      @@logger.warn( "message || socket: nil\n")
      return
    end

    if SocketManager.duplicate?( name, message)
      @@logger.warn( "send: duplicate message dropped")
      return
    end

    # write message
    @@logger.warn( "message send on #{name}: #{message}")
    socket.writeAPRSMessage( message)

    SocketManager.addDuplicate( name, message)

    return
  end


  def SocketManager.addDuplicate( name, message, now = Time.now)
    @@logger.info( "addDuplicate( name, message, now)")

    duplicatePeriod = 0

    socket = @@sockets[name]

    if !socket.nil?
      duplicatePeriod = socket.duplicatePeriod
    end

    duplicates = @@duplicates[name]

    if duplicatePeriod
      # remove old entries from duplicate list
      duplicates.each{ |key, timestamp|
        if now - timestamp > duplicatePeriod
          duplicates.delete( key)
        end
      }
    end
    
    # add message to duplicate list
    duplicates[message.duplicate_key] = now

    return
  end


  def SocketManager.duplicate?( name, message, now = Time.now)
    @@logger.info( "duplicate?( name, message, now)")
    
    duplicatePeriod = 0

    socket = @@sockets[name]

    if !socket.nil?
      duplicatePeriod = socket.duplicatePeriod
    end

    # duplicate check (if enabled)
    if duplicatePeriod 
      duplicates = @@duplicates[name]
      
      timestamp = duplicates[message.duplicate_key]
      
      # drop if timestamp exists and message is a duplicate
      if timestamp && (now - timestamp < duplicatePeriod)
        return true
      end
    end

    return false
  end

end

==> aprs4r-0.9.1.tar.gz: aprs4r-0.9.1/src/aprs4r/SocketManager.rb <==
#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'Logger'

require 'aprs4r/AX25Socket'
require 'aprs4r/ISSocket'
require 'aprs4r/RawSocket'
require 'aprs4r/SocketThread'


class SocketManager

  @@singleton = nil

  @@threads = Array.new
  @@sockets = Hash.new
  @@listeners = Hash.new
  @@duplicates = Hash.new

  @@recvCount = Hash.new
  @@sendCount = Hash.new
  @@lastMessage = nil


  @@logger = Logger.get_logger( "SocketManager")


  def SocketManager.init( configuration)
    @@logger.info( "init( configuration)")

    # dirty hack
    @@recvCount["rf0"] = 0 
    @@sendCount["rf0"] = 0
    @@recvCount["is0"] = 0 
    @@sendCount["is0"] = 0 

    sockets = configuration.devices

    if sockets.nil?
      return
    end

    sockets.each { |name, configuration|

      if configuration.enable 

        @@logger.debug( "name: #{name}")
        @@logger.debug( "configuration: #{configuration}")
        
        socket = nil

        # FIXME: dynamic class loading
        if name == "is0"
          socket = ISSocket.new( configuration)
        elsif name == "rf0"
          socket = AX25Socket.new( configuration)
        end
        
        @@logger.debug( "socket: #{socket}")
        
        @@threads << Thread.new { 
          thread = SocketThread.new( socket)
          thread.run
        }
        
        @@sockets[name] = socket
        @@listeners[name] = Array.new
        @@duplicates[name] = Hash.new

        @@recvCount[name] = 0
        @@sendCount[name] = 0
      end
    }

    return
  end

  
  def SocketManager.addListener( name, listener)
    @@logger.info( "addListener( name, listener)")
    
    if @@sockets.key?( name) 
      listeners = @@listeners[name]
      listeners << listener
    end

    return
  end


  def SocketManager.removeListener( name, listener)
    @@logger.info( "removeListener( name, listener)")

    if @@sockets.key?( name)
      listeners = @@listeners[name]
      listeners.delete( listener)
    end

    return
  end

  
  def SocketManager.receiveAPRSMessage( name, message)
    @@logger.info( "receiveAPRSMessage( name, message)")

    if message.nil?
      return
    end

    if SocketManager.duplicate?( name, message)
      @@logger.warn( "recv: duplicate message dropped")
      return
    end

    @@logger.warn( "message recv on #{name}: #{message}")

    @@recvCount[name] += 1
    @@lastMessage = message

    # deliver message to all registered listeners
    listeners = @@listeners[name]

    if ! listeners.nil?
      listeners.each{ |listener|
        listener.receiveAPRSMessage( message.clone)
      }
    end

    return
  end

  
  def SocketManager.sendAPRSMessage( name, message)
    @@logger.info( "sendAPRSMessage( name, message)")
    
    socket = @@sockets[name]

    if message.nil? || socket.nil?
      @@logger.warn( "message || socket: nil\n")
      return
    end

    if SocketManager.duplicate?( name, message)
      @@logger.warn( "send: duplicate message dropped")
      return
    end

    @@sendCount[name] += 1

    # write message
    @@logger.warn( "message send on #{name}: #{message}")
    socket.writeAPRSMessage( message)

    SocketManager.addDuplicate( name, message)

    return
  end


  def SocketManager.addDuplicate( name, message, now = Time.now)
    @@logger.info( "addDuplicate( name, message, now)")

    duplicatePeriod = 0

    socket = @@sockets[name]

    if !socket.nil?
      duplicatePeriod = socket.duplicatePeriod
    end

    duplicates = @@duplicates[name]

    if duplicatePeriod
      # remove old entries from duplicate list
      duplicates.each{ |key, timestamp|
        if now - timestamp > duplicatePeriod
          duplicates.delete( key)
        end
      }
    end
    
    # add message to duplicate list
    duplicates[message.duplicate_key] = now

    return
  end


  def SocketManager.duplicate?( name, message, now = Time.now)
    @@logger.info( "duplicate?( name, message, now)")
    
    duplicatePeriod = 0

    socket = @@sockets[name]

    if !socket.nil?
      duplicatePeriod = socket.duplicatePeriod
    end

    # duplicate check (if enabled)
    if duplicatePeriod 
      duplicates = @@duplicates[name]
      
      timestamp = duplicates[message.duplicate_key]
      
      # drop if timestamp exists and message is a duplicate
      if timestamp && (now - timestamp < duplicatePeriod)
        return true
      end
    end

    return false
  end

  
  def SocketManager.recvCount( name)
    return @@recvCount[name]
  end

  def SocketManager.setrecvCount( name, count)
    @@recvCount[name] = count
  end
  
  def SocketManager.sendCount( name)
    return @@sendCount[name]
  end

  def SocketManager.setsendCount( name, count)
    @@sendCount[name] = count
  end

  def SocketManager.lastMessage
    return @@lastMessage
  end

end
